package ru.dbotthepony.mc.otm.capability.android;

import net.minecraft.ChatFormatting;
import net.minecraft.network.chat.Component;
import net.minecraft.network.chat.TranslatableComponent;
import net.minecraft.world.entity.player.Player;
import net.minecraft.world.item.ItemStack;

import java.util.ArrayList;
import java.util.List;

public record AndroidFeatureResearchCost(int experience, ItemStack ...items) {
	public boolean matches(Player ply) {
		if (ply.experienceLevel < experience)
			return false;

		for (var stack : items) {
			boolean hit = false;

			if (stack.getTag() == null) {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount()) {
						hit = true;
						break;
					}
				}
			} else {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount() && inv_stack.getTag() != null && inv_stack.getTag().equals(stack.getTag())) {
						hit = true;
						break;
					}
				}
			}

			if (!hit)
				return false;
		}

		return true;
	}

	public List<Component> getTooltip() {
		var list = new ArrayList<Component>();

		if (experience > 0)
			list.add(new TranslatableComponent("otm.android_station.research.xp_cost", experience));

		for (var itemstack : items)
			list.add(new TranslatableComponent("otm.android_station.research.item", itemstack.getDisplayName(), itemstack.getCount()));

		return list;
	}

	public List<Component> getTooltip(Player ply) {
		var list = new ArrayList<Component>();

		if (experience > 0) {
			if (ply.experienceLevel >= experience) {
				list.add(new TranslatableComponent("otm.android_station.research.xp_cost", experience).withStyle(ChatFormatting.DARK_GREEN));
			} else {
				list.add(new TranslatableComponent("otm.android_station.research.xp_cost", experience).withStyle(ChatFormatting.DARK_RED));
			}
		}

		for (var stack : items) {
			boolean hit = false;

			if (stack.getTag() == null) {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount()) {
						hit = true;
						break;
					}
				}
			} else {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount() && inv_stack.getTag() != null && inv_stack.getTag().equals(stack.getTag())) {
						hit = true;
						break;
					}
				}
			}

			if (hit) {
				list.add(new TranslatableComponent("otm.android_station.research.item", stack.getDisplayName(), stack.getCount()).withStyle(ChatFormatting.DARK_GREEN));
			} else {
				list.add(new TranslatableComponent("otm.android_station.research.item", stack.getDisplayName(), stack.getCount()).withStyle(ChatFormatting.DARK_RED));
			}
		}

		return list;
	}

	public boolean research(Player ply) {
		if (!matches(ply))
			return false;

		if (experience > 0)
			ply.giveExperienceLevels(-experience);

		for (var stack : items) {
			if (stack.getTag() == null) {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount()) {
						inv_stack.shrink(stack.getCount());
						break;
					}
				}
			} else {
				for (var inv_stack : ply.getInventory().items) {
					if (inv_stack.is(stack.getItem()) && inv_stack.getCount() >= stack.getCount() && inv_stack.getTag() != null && inv_stack.getTag().equals(stack.getTag())) {
						inv_stack.shrink(stack.getCount());
						break;
					}
				}
			}
		}

		ply.getInventory().setChanged();

		return true;
	}
}
